# $Id$

# will pull this back from the db in time as part of __init__
_columnnames = ["ipFROM", "ipTO", "countrySHORT", "countryLONG", "ipREGION", "ipCITY", "ipLATITUDE",
                "ipLONGITUDE", "ipISP", "ipDOMAIN" ]

_unicode_columnnames = ["countryLONG", "ipREGION", "ipCITY", "ipISP", "ipDOMAIN" ]

badDB = "Unknown database type"

def _to_unicode(str):
    """In absence of a proper spec for the data, try some common types in order"""
    unicode_types = ["ISO-8859-1", "utf-8", "utf-16"]
    for type in unicode_types:
        try:
            return  unicode(str, type).encode('utf-8')
        except UnicodeDecodeError:
            continue
    raise UnicodeError

def _ip_to_num(ip):
    """Convert dotted quad to ip num as per ip2location's docs"""
    nums = ip.split('.')
    return int(nums[0])*16777216 + int(nums[1])*65536 + int(nums[2])*256 + int(nums[3])    


class ip2location:
    """Manage mapping ip -> location"""
    conn = None;    

    def __init__(self, *args, **kargs):
        """setup db connection
        Params are user, password, host, database and database_type
        """
        self._db_type = kargs['database_type']
        if self._db_type == 'mysql':
            import MySQLdb;
            self.conn = MySQLdb.connect(user = kargs['user'], passwd = kargs['password'],
                                host = kargs['host'], db = kargs['database'])
        elif self._db_type == 'postgres':
            import psycopg
            self.conn = psycopg.connect(user = kargs['user'], password = kargs['password'],
                                         host = kargs['host'], database = kargs['database']);
        else:
            raise badDB
        
        
    def __del__(self):
        if self.conn:
            self.conn.close()


    def _db_get(self, attrname, ip):
        if attrname in _columnnames:
            cursor = self.conn.cursor()
            ipnum = _ip_to_num(ip)
            if self._db_type == 'postgres':
                query = "SELECT %s from IPCITYLATLONGISPDOMAIN where ipFROM <= %d AND ipTO >= %d" % (attrname, ipnum, ipnum)
            else:
                query = "SELECT %s from IPCITYLATLONGISPDOMAIN where ipTO >= %d limit 1" % (attrname, ipnum)
            cursor.execute(query)
            row = cursor.fetchone()
            cursor.close()
            if row == None:
                print "Ip not found"
                return None
            if attrname in _unicode_columnnames:
                return _to_unicode(row[0])
            else:
                return row[0]
        else:
            raise AttributeError, attrname

    def all(self, ip):
        cursor = self.conn.cursor()
        ipnum = _ip_to_num(ip)
        if self._db_type == 'postgres':
            query = "SELECT * from IPCITYLATLONGISPDOMAIN where ipFROM <= %d and ipTO >= %d" % (ipnum, ipnum)
        else:
            query = "SELECT * from IPCITYLATLONGISPDOMAIN where ipTO >= %d limit 1" % ipnum
        cursor.execute(query)
        row = cursor.fetchone()
        cursor.close()
        if row == None:
            return None
        return { 'country'   : _to_unicode(row[3]),
                 'region'    : _to_unicode(row[4]),
                 'city'      : _to_unicode(row[5]),
                 'latitude'  : row[6],
                 'longitude' : row[7],
                 'isp'       : _to_unicode(row[8]),
                 'domain'    : _to_unicode(row[9])  }

    def city(self, ip):
        """return city for a given IP"""
        return self._db_get("ipCITY", ip)

    def country(self, ip):
        """return country for a given IP"""
        return self._db_get("countryLONG", ip)

    def domain(self, ip):
        """return domain for a given IP"""
        return self._db_get("ipDOMAIN", ip)

    def isp(self, ip):
        """return ISP for a given IP"""
        return self._db_get("ipISP", ip)

    def latitude(self, ip):
        """return latitude as double for a given IP"""
        return self._db_get("ipLATITUDE", ip)

    def longitude(self, ip):
        """return longitude as double for a given IP"""
        return self._db_get("ipLONGITUDE", ip)

    def region(self, ip):
        """return region for a given IP"""
        return self._db_get("ipREGION", ip)
       

if __name__ == "__main__":
    ipdb = ip2location(host="localhost", user = "ip", password = "password",
                       database = "ip2location", database_type='mysql')
    city = ipdb.city("144.32.226.226")
    assert(city == "YORK")
    city = ipdb.city("213.66.43.35")
    assert(city == 'LINK\xc3\xb6PING')
    assert(ipdb.all("444.43.32.23") == None)

